
/* Chrysalide - Outil d'analyse de fichiers binaires
 * restricted.c - prototypes pour l'équivalent Python du fichier "analysis/contents/restricted.c"
 *
 * Copyright (C) 2015-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "restricted.h"


#include <pygobject.h>


#include <i18n.h>


#include <analysis/contents/restricted.h>


#include "../content.h"
#include "../../access.h"
#include "../../helpers.h"
#include "../../arch/vmpa.h"



/* Crée un nouvel objet Python de type 'BinContent'. */
static PyObject *py_restricted_content_new(PyTypeObject *, PyObject *, PyObject *);



/******************************************************************************
*                                                                             *
*  Paramètres  : type = type de l'objet à instancier.                         *
*                args = arguments fournis à l'appel.                          *
*                kwds = arguments de type key=val fournis.                    *
*                                                                             *
*  Description : Crée un nouvel objet Python de type 'BinContent'.            *
*                                                                             *
*  Retour      : Instance Python mise en place.                               *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static PyObject *py_restricted_content_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
    PyObject *result;                       /* Instance à retourner        */
    PyObject *content_obj;                  /* Objet pour le contenu       */
    PyObject *range_obj;                    /* Objet pour la restriction   */
    int ret;                                /* Bilan de lecture des args.  */
    GBinContent *content;                   /* Instance GLib correspondante*/
    mrange_t *range;                        /* Restriction à appliquer     */
    GBinContent *restricted;                /* Création GLib à transmettre */

    ret = PyArg_ParseTuple(args, "OO", &content_obj, &range_obj);
    if (!ret) return NULL;

    ret = PyObject_IsInstance(content_obj, (PyObject *)get_python_binary_content_type());
    if (!ret)
    {
        PyErr_SetString(PyExc_TypeError, _("The first argument must be an instance of BinContent."));
        return NULL;
    }

    ret = PyObject_IsInstance(range_obj, (PyObject *)get_python_mrange_type());
    if (!ret)
    {
        PyErr_SetString(PyExc_TypeError, _("The second argument must be an instance of mrange."));
        return NULL;
    }

    content = G_BIN_CONTENT(pygobject_get(content_obj));

    range = get_internal_mrange(range_obj);

    restricted = g_restricted_content_new(content, range);

    result = pygobject_new(G_OBJECT(restricted));

    return result;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : -                                                            *
*                                                                             *
*  Description : Fournit un accès à une définition de type à diffuser.        *
*                                                                             *
*  Retour      : Définition d'objet pour Python.                              *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

PyTypeObject *get_python_restricted_content_type(void)
{
    static PyMethodDef py_restricted_content_methods[] = {
        { NULL }
    };

    static PyGetSetDef py_restricted_content_getseters[] = {
        { NULL }
    };

    static PyTypeObject py_restricted_content_type = {

        PyVarObject_HEAD_INIT(NULL, 0)

        .tp_name        = "pychrysalide.analysis.contents.RestrictedContent",
        .tp_basicsize   = sizeof(PyGObject),

        .tp_flags       = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,

        .tp_doc         = "PyChrysalide binary restricted content",

        .tp_methods     = py_restricted_content_methods,
        .tp_getset      = py_restricted_content_getseters,
        .tp_new         = py_restricted_content_new

    };

    return &py_restricted_content_type;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : module = module dont la définition est à compléter.          *
*                                                                             *
*  Description : Prend en charge l'objet 'pychrysalide.....RestrictedContent'.*
*                                                                             *
*  Retour      : Bilan de l'opération.                                        *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

bool ensure_python_restricted_content_is_registered(void)
{
    PyTypeObject *type;                     /* Type Python 'Restricted...' */
    PyObject *module;                       /* Module à recompléter        */
    PyObject *dict;                         /* Dictionnaire du module      */

    type = get_python_restricted_content_type();

    if (!PyType_HasFeature(type, Py_TPFLAGS_READY))
    {
        module = get_access_to_python_module("pychrysalide.analysis.contents");

        dict = PyModule_GetDict(module);

        if (!register_class_for_pygobject(dict, G_TYPE_RESTRICTED_CONTENT, type, &PyGObject_Type))
            return false;

    }

    return true;

}
